#version 330 core

// Input texture
uniform sampler2D inputTexture;

// Texture resolution (automatically provided from the input texture)
uniform vec2 textureResolution; // Passed from the application (width, height)

// Aspect ratios
const float originalAspectRatio = 16.0 / 9.0; // 16:9
const float targetAspectRatio = 2.35; // 2.35:1 scope screen

in vec2 TexCoords;  // Texture coordinates passed from vertex shader
out vec4 FragColor; // Output color

// Non-linear stretch function for NLS effect
vec2 nonLinearStretch(vec2 texCoords, float stretchFactor) {
    float x = texCoords.x;

    // Apply the stretch more gradually toward the sides, using an easing function
    if (x < 0.5) {
        // Left side: stretch gradually from the center
        x = 0.5 * (1.0 - cos(x * 3.14159265359 * 0.5));
    } else {
        // Right side: stretch gradually from the center
        x = 0.5 + 0.5 * (1.0 - cos((x - 0.5) * 3.14159265359 * 0.5));
    }

    // Apply the overall stretch factor
    texCoords.x = texCoords.x * stretchFactor;
    
    return texCoords;
}

void main() {
    // Get the aspect ratio of the input texture (based on its resolution)
    float textureAspectRatio = textureResolution.x / textureResolution.y;

    // Calculate the aspect ratio difference (how much stretching is required)
    float aspectDiff = targetAspectRatio / originalAspectRatio;

    // Apply the non-linear stretch to the texture coordinates
    vec2 stretchedTexCoords = nonLinearStretch(TexCoords, aspectDiff);

    // Fetch the color from the texture
    vec4 color = texture(inputTexture, stretchedTexCoords);

    // Output the final color
    FragColor = color;
}
