#version 330 core

// Input texture
uniform sampler2D inputTexture;

// Texture resolution (automatically provided from the input texture)
uniform vec2 textureResolution; // Passed from the application (width, height)

// Aspect ratios
const float originalAspectRatio = 16.0 / 9.0; // 16:9
const float targetAspectRatio = 2.35; // 2.35:1 scope screen

in vec2 TexCoords;  // Texture coordinates passed from vertex shader
out vec4 FragColor; // Output color

// Non-linear stretch function
vec2 nonLinearStretch(vec2 texCoords, float aspectDiff) {
    // Stretch the texture coordinates based on the aspect difference
    if (texCoords.x < 0.5) {
        // Stretch the left side more smoothly
        texCoords.x = texCoords.x * aspectDiff;
    } else {
        // Stretch the right side more smoothly
        texCoords.x = 0.5 + (texCoords.x - 0.5) * aspectDiff;
    }

    return texCoords;
}

void main() {
    // Get the aspect ratio of the input texture (based on its resolution)
    float textureAspectRatio = textureResolution.x / textureResolution.y;

    // Calculate the aspect ratio difference (how much stretching is required)
    float aspectDiff = targetAspectRatio / originalAspectRatio;

    // Apply the non-linear stretch to the texture coordinates
    vec2 stretchedTexCoords = nonLinearStretch(TexCoords, aspectDiff);

    // Fetch the color from the texture
    vec4 color = texture(inputTexture, stretchedTexCoords);

    // Output the final color
    FragColor = color;
}
